package org.greenstone.gsdl3.gs3build.util;

import java.util.*;

public class HTMLBlock
{ Vector content;
  int    tagpos;
  int    endpos;

  public HTMLBlock(int docpos)
  { this.content = new Vector(1);
    this.tagpos  = docpos;
  }

  public HTMLBlock(int docpos, int endpos)
  { this.content = new Vector(1);
    this.tagpos  = docpos;
    this.endpos  = endpos;
  }

  public void addTag(HTMLTag tag)
  { if (tag.endPos() > this.endpos)
    { this.endpos = tag.endPos();
    }
    content.addElement(tag);
  }

  /**
   *  @deprecccate
  public void addText(String text)
  { content.addElement(text);
  }
   */

  public void addText(HTMLCText text)
  { if (text.endPos() > this.endpos)
    { this.endpos	= text.endPos();
    }
    content.addElement(text);
  }

  /**
   * @return HTML encoded <code>String</code> of the document
   */
  public String contentString()
  { int member;
    StringBuffer reply;
    String type;

    reply = new StringBuffer();
    for (member = 1; member < content.size() - 1; member ++)
    { /*if (content.elementAt(member) instanceof String)
      { reply.append((String) content.elementAt(member));
      }
      else 
      */
      if (content.elementAt(member) instanceof HTMLCText)
      {	reply.append(((HTMLCText) content.elementAt(member)).toString());
      }
    }
    return reply.toString();
  }

  /**
   * Returns document position of start of the block
   */
  public int startPos()
  { return this.tagpos;
  }

  /**
   * Returns document position of end of the block
   */
  public int endPos()
  { return this.tagpos;
  }

  /**
   * @return head (first) tag of the block
   */
  public HTMLTag headTag()
  { return ((HTMLTag) content.elementAt(0));
  }

  /**
   * @return name (as a <code>String</code>) of the tag at the head of the block
   */
  public String headTagName()
  { return ((HTMLTag) content.elementAt(0)).tagName();
  }

  /**
   * @return name of the tail (last) tag of the block as a <code>String</code>
   */
  public String tailTagName()
  { return (((HTMLTag) content.elementAt(content.size() - 1)).tagName());
  }
	
	
  /**
   *  @return: starting element character position of the item-th item
   */
  private int elementStartPos(int item)
  { Object element;
    int		pos = -1;
		
    element = this.content.elementAt(item);
    if (element instanceof HTMLTag)
    {	pos	= ((HTMLTag) element).startPos();
    }
    else if (element instanceof HTMLCText)
    {	pos	= ((HTMLCText) element).startPos();
    }
    return pos;		
  }
	
  /**
   *  @return: ending element character position of the item-th item
   */
  private int elementEndPos(int item)
  { Object	element;
    int		pos = -1;
		
    element = this.content.elementAt(item);
    if (element instanceof HTMLTag)
    { pos = ((HTMLTag) element).endPos();
    }
    else if (element instanceof HTMLCText)
    { pos = ((HTMLCText) element).endPos();
    }
    return pos;		
  }

  /**
   *  @return the HTMLBlock of the indicated subitems
   */
  public HTMLBlock subBlock(int startitem, int enditem)
  { int       i;
    Object    element;
    HTMLBlock reply;
    reply = new HTMLBlock(this.elementStartPos(startitem),
			  this.elementEndPos(enditem-1));
    // copy all the bits
    for (i = startitem; i < enditem; i++)
    { element = this.content.elementAt(i);
      if (element instanceof HTMLTag)
      { reply.addTag((HTMLTag) element);
      }
      else if (element instanceof HTMLCText)
      {	reply.addText((HTMLCText) element);
      }
      /*
	else if (element instanceof String)
	{	reply.addText((String) element);
	}
      */
    }
    return reply;
  }

  /**
   * Return the position of the given <code>HTMLBlock</code> within this block
   * @return the tag index of the child block; or < 0 if not a chile
   */
  public int find(HTMLBlock subblock)
  { int start, end, at, attag;
    HTMLTag tag;

    start = 0;
    end		= this.content.size();
    while (start != end)
    { /* compare positions */
      at = (start + end) / 2;
      attag	= at;
      while (attag < end &&
	     (content.elementAt(attag) instanceof HTMLTag) == false)
      { attag ++;
      }

      if (attag == end)
      { end = at;
        continue;
      }
      tag =	(HTMLTag) content.elementAt(attag);
      if (tag.startPos() > subblock.startPos())
      { end = at;				
      }
      else if (tag.startPos() < subblock.startPos())
      { start	= attag + 1;
      }
      else
      {	return attag;
      }
    }
    return -1;
  }

  /**
   * @return The raw HTML of the block
   */
  public String HTMLString()
  { int 					loop;
    StringBuffer	reply;

    reply	= new StringBuffer();
    for (loop = 0; loop < this.content.size(); loop ++)
    { /*
	if (content.elementAt(loop) instanceof String)
      {	reply.append((String) content.elementAt(loop));
      }
      else 
      */
      if (content.elementAt(loop) instanceof HTMLCText)
      {	reply.append(((HTMLCText) content.elementAt(loop)).toString());
      }
      else if (content.elementAt(loop) instanceof HTMLTag)
      {	reply.append(((HTMLTag) content.elementAt(loop)).toString());
      }
    }
    return reply.toString();
  }

  /**
   * @return The text of the block - no HTML tags.  If no text is present, any
   *         <code>alt</code> information for <code>img</code> tags will be given 
   *         instead
   */
  public String toString()
  { int member;
    StringBuffer reply;
    StringBuffer ireply;
    
    reply = new StringBuffer("");
    ireply = new StringBuffer("");
    for (member = 0; member < content.size(); member ++)
    { /*if (content.elementAt(member) instanceof String)
      { reply.append((String) content.elementAt(member));
      }
      */
      if (content.elementAt(member) instanceof HTMLCText)
      { reply.append(((HTMLCText) content.elementAt(member)).toString());
      }
      else if (content.elementAt(member) instanceof HTMLTag)
      { if (((HTMLTag) content.elementAt(member)).tagName().equals("img"))
	{ ireply.append(((HTMLTag) content.elementAt(member)).idValue("alt"));
	}
      }
    }
    reply = HTMLCText.cleanString(reply);
    ireply = HTMLCText.cleanString(ireply);
//		reply = reply.trim();
    if (reply == null || reply.length() == 0)
    { return ireply.toString();
    }
    return reply.toString();
  }

  public HTMLObject elementAt(int at)
  { return (HTMLObject) this.content.elementAt(at);
  }

  /**
   * @return the number of elements in the block
   */
  public int size()
  { return this.content.size();
  }

  /**
   * @return an enumeration of all elements in the block
   */
  public Enumeration elements()
  { Enumeration enum;
  
    enum = new HTMLBlockEnumerator(this);
    return enum;
  }

  /**
   * @return an enumeration of all tags in the block
   */
  public Enumeration tagElements()
  { Enumeration enum;

    enum = new HTMLBlockTagEnumerator(this);
    return enum;
  }

  /**
   * Finalization method
   */
  protected void finalize() throws Throwable
  { this.content	= null;
    super.finalize();
  }
}

final class HTMLBlockEnumerator implements Enumeration
{ private HTMLBlock block;
	int			member;

	public HTMLBlockEnumerator(HTMLBlock block)
	{ this.block	= block;
		this.member	= 0;
	}

	public Object nextElement()
	{ Object element;

		element = this.block.content.elementAt(member);
		member ++;
		return element;
	}

	public boolean hasMoreElements()
	{ if (block == null || this.member == this.block.content.size())
		{ return false;
		}
		return true;
	}
}

final class HTMLBlockTagEnumerator implements Enumeration
{ private HTMLBlock block;
	int			member;

	public HTMLBlockTagEnumerator(HTMLBlock block)
	{ this.block	= block;
		this.member	= 0;
		while ( this.member < this.block.content.size() &&
						this.block.content.elementAt(this.member) instanceof HTMLTag == false)
		{	this.member ++;
		}
	}

	public Object nextElement()
	{ Object element;

		element = this.block.content.elementAt(member);
		do
		{ member ++;
		} while (this.block.content.elementAt(member) instanceof HTMLTag == false);
		return element;
	}

	public boolean hasMoreElements()
	{ if (block == null || this.member == this.block.content.size())
		{ return false;
		}
		return true;
	}
}

